@@ -19,7 +19,7 @@
 ;; URL:              https://github.com/meedstrom/el-job
 ;; Created:          2024-10-30
 ;; Keywords:         processes
-;; Package-Version:  2.4.4
+;; Package-Version:  2.4.5
 ;; Package-Requires: ((emacs "29.1"))
 
 ;;; Commentary:
@@ -74,9 +74,9 @@
   "Maybe pass FMT and ARGS to `message'.
 LEVEL is the threshold that `el-job--debug-level' should meet or exceed
 to unlock this message."
-  (declare (indent 2))
+  (declare (indent 1))
   (when (<= level el-job--debug-level)
-    (apply #'message fmt args)))
+    (apply #'message (concat "el-job: " fmt) args)))
 
 (defun el-job--locate-lib-in-load-history (feature)
   "Look for the .eln, .elc or .el file corresponding to FEATURE.
@@ -115,9 +115,10 @@ an .eln anyway, without your having to recompile on save."
                                          (symbol-function (cdr elem))))))
                               ;; FIXME: comp sometimes deletes old eln during
                               ;; recompilation, but does not load the new eln,
-                              ;; at least not in a way that updates
-                              ;; `load-history'.  Current workaround is return
-                              ;; nil, ie fall back on FILE; not ideal.
+                              ;; at least not in a way that updates the
+                              ;; `native-comp-unit-file'.  Current workaround
+                              ;; is return nil, ie fall back on FILE; not
+                              ;; ideal.
                               (when (file-exists-p eln)
                                 eln))))
        file)))
@@ -170,7 +171,7 @@ find the correct file."
                     `(lambda (&rest _) ,elc)))
               (when (native-comp-available-p)
                 ;; FIXME: Guix overrides `comp-el-to-eln-rel-filename' to
-                ;; output filenames with NO HASH!  So compiling now can result
+                ;; output filenames with no hash!  So compiling now can result
                 ;; in an .eln in ~/.emacs.d/ that will always take precedence
                 ;; over the one shipped by Guix.  If we want to cover for that,
                 ;; it'd be safer to compile into /tmp with a filename based on
@@ -180,7 +181,7 @@ find the correct file."
                 (native-compile-async (list loaded)))
               ;; Native comp may take a while, so build and return .elc this
               ;; time.  We should not pick a preexisting .elc from `load-path'
-              ;; if Emacs is now running interpreted code, since that currently
+              ;; if Emacs is running interpreted code now, since that currently
               ;; running code is likely newer.
               (if (or (file-newer-than-file-p elc loaded)
                       (byte-compile-file loaded))
@@ -382,6 +383,8 @@ The subprocesses do not inherit `load-path', it is the current Emacs
 process that locates files \(by inspecting `load-history', via
 `el-job--ensure-compiled-lib'), then gives them to each subprocess.
 
+Tip: To let them inherit `load-path' anyway, add it to INJECT-VARS.
+
 
 INPUTS is a list that will be split by up to the output of
 `num-processors', and this determines how many subprocesses will spawn.
@@ -395,7 +398,8 @@ If INPUTS returns nil, do nothing and return the symbol
 
 The subprocesses have no access to current Emacs state.  The only way
 they can affect current state, is if FUNCALL-PER-INPUT returns data,
-which is then handled by CALLBACK function in the current Emacs.
+which is then handled by CALLBACK function in the current Emacs, as
+described earlier.
 
 Emacs stays responsive to user input up until all subprocesses finish,
 which is when their results are merged and CALLBACK is executed.
@@ -481,10 +485,12 @@ For debugging, see these commands:
                 (setf .spawn-args new-spawn-args)
                 (el-job--dbg 2 "New arguments, resetting processes for %s" id)
                 (setq do-respawn t)))
-            (when do-respawn
-              (el-job--disable job)
-              (apply #'el-job--spawn-processes .spawn-args))
-            (el-job--exec-workload job)))))))
+            (let ((error-maybe
+                   (when do-respawn
+                     (el-job--disable job)
+                     (apply #'el-job--spawn-processes .spawn-args))))
+              (unless error-maybe
+                (el-job--exec-workload job)))))))))
 
 (defvar-local el-job-here nil)
 (defun el-job--spawn-processes (job load-features inject-vars funcall-per-input)
@@ -513,7 +519,8 @@ see `el-job-launch'."
            "--eval" (format "(el-job-child--work #'%S)" funcall-per-input)))
          ;; Ensure the working directory is not remote.
          ;; https://github.com/meedstrom/org-node/issues/46
-         (default-directory invocation-directory))
+         (default-directory invocation-directory)
+         return-value)
     (el-job--with job (.stderr .id .ready .spawn-args .n-cores-to-use)
       (setf .stderr
             (with-current-buffer
@@ -546,7 +553,12 @@ see `el-job-launch'."
         ;; https://github.com/meedstrom/org-node/issues/75
         (( file-error )
          (el-job--disable job)
-         (el-job--dbg 1 "el-job: Terminated job because of: %S" err))))))
+         (el-job--dbg 1 "Terminated job because of: %S" err)
+         (setq return-value err)))
+      ;; Return non-nil on error, so caller can choose to fail quietly.
+      ;; We suppressed the error signal because for some users, it only occurs
+      ;; intermittently and does not break things.
+      return-value)))
 
 (defun el-job--exec-workload (job)
   "Split the queued inputs in JOB and pass to all children.
@@ -627,9 +639,13 @@ after a short delay.  N is the count of checks done so far."
                     (run-with-timer
                      (/ n 32.0) nil #'el-job--poll (1+ n) busy-bufs))
             (el-job--disable el-job-here)
-            (message "el-job: Timed out, was busy for 30+ seconds: %s"
-                     (el-job-id el-job-here)))
-        (cl-assert (not (member (el-job-timer el-job-here) timer-list)))
+            (el-job--dbg 0 "Timed out, was busy for 30+ seconds: %s"
+                         (el-job-id el-job-here)))
+        (when (member (el-job-timer el-job-here) timer-list)
+          ;; It does, somehow, happen
+          ;; https://github.com/meedstrom/org-node/issues/94
+          (el-job--dbg 1 "Timer still active (this is a bug), recovering")
+          (cancel-timer (el-job-timer el-job-here)))
         (setf (el-job-timer el-job-here)
               (run-with-timer 30 nil #'el-job--reap (current-buffer)))))))
 
@@ -640,15 +656,16 @@ after a short delay.  N is the count of checks done so far."
       (el-job--disable job)
       (el-job--dbg 2 "Reaped idle processes for %s" (el-job-id job)))))
 
-;; TODO: Consider using `with-local-quit', because this fires on a timer
+;; TODO: Consider using `with-local-quit' because this fires on a timer, which
+;;       means `inhibit-quit' is t, but it'd be good to allow quitting out of a
+;;       hung or long-running CALLBACK function.
 (defun el-job--handle-output ()
   "Handle output in current buffer.
 
 If this is the last output for the job, merge all outputs, maybe execute
 the callback function, finally maybe run the job again if there is now
 more input in the queue."
-  (let ((inhibit-quit t)
-        (proc (get-buffer-process (current-buffer)))
+  (let ((proc (get-buffer-process (current-buffer)))
         (job el-job-here)
         finish-time
         durations
@@ -685,6 +702,11 @@ more input in the queue."
         ;; Somehow, it took 700 lines of code to get here.
         (when .callback
           (funcall .callback (el-job--zip-all .result-sets) job))
+        ;; REVIEW: Either
+        ;; 1. get rid of this code path (see comments in `el-job-launch' re.
+        ;;    queued inputs); or
+        ;; 2. consider if we need to check length of .queued-inputs to maybe
+        ;;    respawn all processes (see what `el-job-launch' does).
         (when .queued-inputs
           (el-job--exec-workload job))))))
 
@@ -819,3 +841,9 @@ Safely return nil otherwise, whether or not ID is known."
 (provide 'el-job)
 
 ;;; el-job.el ends here
+
+;; Local Variables:
+;; checkdoc-spellcheck-documentation-flag: nil
+;; checkdoc-verb-check-experimental-flag: nil
+;; emacs-lisp-docstring-fill-column: 72
+;; End:
